#include <Arduino.h>
#include <SPI.h>
#include <Ethernet2.h>
//
#include "LanConfiguration.h"
#include "LedSystem.h"
//
/*
 Communication over Browser-Line:
   http://192.168.178.177/LSO
   http://192.168.178.177/LSF
   http://192.168.178.177/LSB_10_200_100
*/
//
const int PIN_LEDSYSTEM = 2;
//
EthernetServer Esp32Server(IPPORT_HTTP);
int HttpRequestCounter = 0;
String HtmlHeader = "";
char* PCommand = (char*)"";
char* PParameters[4];
int ParameterCount = 0;
//
CLedSystem LedSystem(PIN_LEDSYSTEM);
//
void setup() 
{
  Serial.begin(115200);
  delay(333);
  Serial.println("\r\n");
  Serial.println("*****************************");
  Serial.println("* Esp32LanCommandDispatcher *");
  Serial.println("* Version: 01V05            *");
  Serial.println("* Date...: 211016           *");
  Serial.println("* Time...: 0757            *");
  Serial.println("* Author.: OMdevelop        *");
  Serial.println("*****************************");
  //
  LedSystem.Open();
  for (int RI = 0; RI < 10; RI++)
  {
    LedSystem.SwitchOn();
    delay(100);
    LedSystem.SwitchOff();
    delay(100);
  }
  //
  Serial.println("Init Lan-Connection...");
  Ethernet.init(PIN_SPI_CSS);  
  Ethernet.begin(MAC_ESP32W5500, 
                 IPADDRESS_DEVICE); 
//                 IPADDRESS_DNS,    
//                 IPADDRESS_GTW,    
//                 IPADDRESS_SNM);   
  Esp32Server.begin(IPPORT_HTTP);
  Serial.print("Connected to IPAddress[");
  Serial.print(Ethernet.localIP());
  Serial.println("]");
  //
  delay(200);
}

void loop() 
{
    if (!LedSystem.Execute())
  {
    delay(50);
    // listen for incoming clients
    EthernetClient Esp32Client = Esp32Server.available();
    if (Esp32Client) 
    { // debug Serial.println("Http-Request from Client detected");
      // Http-Request ends with a blank line!
      boolean CurrentLineIsBlank = true;
      while (Esp32Client.connected()) 
      {
        if (Esp32Client.available()) 
        {
          char C = Esp32Client.read();
          // PROTOCOL !!!! Serial.write(C);
          HtmlHeader += C;
          // if you've gotten to the end of the line (received a newline
          // character) and the line is blank, the http request has ended,
          // so you can send a reply
          if (('\n' == C) && CurrentLineIsBlank) 
          { 
            HttpRequestCounter++;
            //-----------------------------------------------------------          
            // Analyse HttpRequest-Parameters:
            //-----------------------------------------------------------
            //      LedSystem - LSF
            if (0 <= HtmlHeader.indexOf("GET /LSF"))
            {
              HtmlHeader = "";
              Serial.println("Parameter[/LSF] - LedSystemOff");
              LedSystem.SwitchOff();
            }
            else // LedSystem - LSO
            if (0 <= HtmlHeader.indexOf("GET /LSO")) 
            {
              HtmlHeader = "";
              Serial.println("Parameter[/LSO] - LedSystemOn");
              LedSystem.SwitchOn();
            }
            else // LedSystem - LSB <count> <period> <width>
            if (0 <= HtmlHeader.indexOf("GET /"))
            {      
              PCommand = (char*)"";
              PParameters[0] = (char*)"";      
              PParameters[1] = (char*)"";      
              PParameters[2] = (char*)"";      
              PParameters[3] = (char*)"";
              //String CommandLine = (char*)(&(HtmlHeader[5]));
              int CommandIndex = HtmlHeader.indexOf("LSB");
              if (0 <= CommandIndex)
              {                          
                char* Token = strtok((char*)(&(HtmlHeader[CommandIndex])), "_ \r\n\t");
                //Serial.print(">>>");
                //Serial.print(Token);
                //Serial.println("<<<");
                if (NULL != Token)
                {
                  PCommand = (char*)(Token);
                }
                ParameterCount = 0;
                Token = strtok(NULL, "_ \r\n\t");
                while (NULL != Token)
                {
                  if (0 == strncmp(Token, "HTTP", 4))
                  {
                    Token = NULL;
                  }
                  else
                  {
                    PParameters[ParameterCount] = Token;
                    Token = strtok(NULL, "_ \r\n\t");
                    if (ParameterCount < 4)
                    {
                      ParameterCount++;
                    } 
                    else
                    {
                      Token = NULL;
                    }
                  }
                }
                Serial.print("C[");
                Serial.print(PCommand);
                Serial.print("] ");
                if (0 < ParameterCount)
                {
                  Serial.print(" P0[");
                  Serial.print(PParameters[0]);
                  Serial.print("] ");
                }
                if (1 < ParameterCount)
                {
                  Serial.print(" P1[");
                  Serial.print(PParameters[1]);
                  Serial.print("] ");
                }
                if (2 < ParameterCount)
                {
                  Serial.print(" P2[");
                  Serial.print(PParameters[2]);
                  Serial.print("] ");
                }
                if (3 < ParameterCount)
                {
                  Serial.print(" P3[");
                  Serial.print(PParameters[3]);
                  Serial.print("] ");
                }
                Serial.println("");
              }        
              if (3 == ParameterCount)
              { // http://192.168.178.177/LSB_10_200_100
                Serial.println("*** BLS ... ***");
                LedSystem.Blink(atol(PParameters[0]), atol(PParameters[1]), atol(PParameters[2]));
              }
              HtmlHeader = "";
            }
            else
            {
  //            Serial.print("Invalid Parameter detected: <");;
  //            Serial.print(HtmlHeader);;
  //            Serial.println(">");;
              HtmlHeader = "";
            }
            //-----------------------------------------------------------          
            // Send a standard HttpResponseHeader back to Clients Request
            //-----------------------------------------------------------          
            Esp32Client.println("HTTP/1.1 200 OK");
            Esp32Client.println("Content-Type: text/html");
            Esp32Client.println("Connection: close");  // the connection will be closed after completion of the response
            Esp32Client.print("Refresh: ");
            Esp32Client.println(TIME_AUTOREFRESH);
            Esp32Client.println("");
            Esp32Client.println("<!DOCTYPE HTML>");
            Esp32Client.println("<html>");
            // WebSite-Header
            Esp32Client.println("<h2>Esp32LanPowerSwitch</h2>");
            // HttpRequestCounter
            Esp32Client.print("<br>HttpRequestCounter[1]: ");
            Esp32Client.print(HttpRequestCounter);
            Esp32Client.println("<br />");
            // LocalDeviceTime
            Esp32Client.print("<br />LocalDeviceTime[ms]: ");
            Esp32Client.print(millis());
            Esp32Client.println("<br />");
            // AdcValues
            Esp32Client.println("<br />ADCValues:<br />");
            for (int AnalogChannel = 0; AnalogChannel < 6; AnalogChannel++) 
            {
              int SensorReading = random(0, 4096); //analogRead(AnalogChannel);
              Esp32Client.print("AnalogInput[");
              Esp32Client.print(AnalogChannel);
              Esp32Client.print("]: ");
              Esp32Client.print(SensorReading);
              Esp32Client.println("<br />");
            }
            // RelayList
            Esp32Client.println("<br />LedSystem:<br />");
            Esp32Client.print("State["); Esp32Client.print(LedSystem.GetState()); Esp32Client.println("]<br />");
            //
            Esp32Client.println("</html>");
            break;
          }
          if ('\n' == C) 
          { // you're starting a new line
            CurrentLineIsBlank = true;
          }
          else 
          if ('\r' != C) 
          { // you've gotten a character on the current line
            CurrentLineIsBlank = false;
          }
        }
      }
      // give the web browser time to receive the data
      delay(1);
      // close the connection:
      Esp32Client.stop();
      // debug Serial.println("Esp32Client disconnected");
    }
  }
}